# encoding: utf-8
"""
@project: djangoModel->statu_code_api
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis:
@created_time: 2022/10/14 14:10
"""
from rest_framework.views import APIView

from ..models import EnrollStatusCode
from ..utils.custom_response import util_response


class OtherListAPIView(APIView):
    def enroll_status_code(self):
        res = list(EnrollStatusCode.objects.all().values())
        return util_response(data=res)



from django.urls import re_path

from .api.enroll_apis import EnrollAPI
from .api.enroll_statistics_apis import EnrollStatisticsAPI
from .api.enroll_status_code_api import EnrollStatusCodeAPI
from .api.rarely_data_list_api import OtherListAPIView
from .api.record_apis import RecordAPI
from .api.rule_apis import RuleAPI
from .api.subitem_apis import SubitemApis
from .api.subitem_record_apis import SubitemRecordApis
from .api.valuation_api import ValuationAPIView
from .service_register import register

# 对服务进行，注册可直接访问，提过流程模块等调度模块调用
register()

urlpatterns = [
    # 报名API
    re_path(r'^list/?$', EnrollAPI.list),
    re_path(r'^own_list/?$', EnrollAPI.own_list),
    re_path(r'^undertake_list/?$', EnrollAPI.undertake_list),
    re_path(r'^detail/?(?P<enroll_id>\d+)?$', EnrollAPI.detail),
    re_path(r'^edit/?(?P<enroll_id>\d+)?$', EnrollAPI.edit),
    re_path(r'^delete/?(?P<enroll_id>\d+)?$', EnrollAPI.delete),
    re_path(r'^add/?$', EnrollAPI.add),
    re_path(r'^enroll/?$', EnrollAPI.enroll),  # 报名记录联合添加报名记录和分项记录, 当前逻辑暂不使用
    re_path(r'^pay_callback/?$', EnrollAPI.enroll_pay_callback),

    # 报名记录
    re_path(r'^record_list/?$', RecordAPI.list),
    re_path(r'^record_list_v2/?$', RecordAPI.list_v2), # 关联较多数据，发起人、报名人、信息表
    re_path(r'^record_add/?$', RecordAPI.add),
    # re_path(r'^record_del/?(?P<pk>\d+)?$', RecordAPI.record_del), // 记录没有删除这个流程节点
    re_path(r'^record_edit/?(?P<pk>\d+)?$', RecordAPI.record_edit),
    re_path(r'^record_detail/?(?P<pk>\d+)?$', RecordAPI.record_detail),
    re_path(r'^appoint/?$', RecordAPI.appoint),  # 指派报名人完成任务

    # 报名分项
    re_path(r'^subitem_add/?$', SubitemApis.add),
    re_path(r'^subitem_detail/?(?P<pk>\d+)?$', SubitemApis.detail),
    re_path(r'^subitem_batch_add/?$', SubitemApis.batch_add),
    re_path(r'^subitem_list/?$', SubitemApis.list),
    re_path(r'^subitem_edit/?(?P<pk>\d+)?$', SubitemApis.edit),
    re_path(r'^subitem_batch_edit/?$', SubitemApis.batch_edit),
    re_path(r'^subitem_edit_by_enroll/?(?P<enroll_id>\d+)?$', SubitemApis.edit_by_enroll_id),  #
    re_path(r'^subitem_extend_field/?$', SubitemApis.extend_field),

    # 报名分项记录
    re_path(r'^subitem_record_add/?$', SubitemRecordApis.add),

    # re_path(r'^subitem_record_del/?(?P<pk>\d+)?$', SubitemRecordApis.delete), // 记录没有删除这个流程节点
    re_path(r'^subitem_record_list/?$', SubitemRecordApis.list),
    re_path(r'^subitem_record_edit/?(?P<pk>\d+)?$', SubitemRecordApis.edit),
    re_path(r'^subitem_record_batch_add/?$', SubitemRecordApis.batch_add),

    # 报名规则
    re_path(r'^rule_list/?$', RuleAPI.list),
    re_path(r'^rule_edit/?(?P<rule_value_id>\d+)?$', RuleAPI.edit),
    re_path(r'^rule_delete/?(?P<rule_value_id>\d+)?$', RuleAPI.delete),
    re_path(r'^rule_add/?$', RuleAPI.add),
    re_path(r'^rule_group_list/?$', RuleAPI.group_list),  # 分组列表

    # 计价接口
    re_path(r'^valuation/?$', ValuationAPIView.valuate),
    re_path(r'^valuation_test/?$', ValuationAPIView.valuate_test),
    re_path(r'^valuation_detailed_list/?(?P<enroll_id>\d+)?$', ValuationAPIView.valuation_detailed_list),

    # 非热更新列表返回
    re_path(r'^status_code/?$', OtherListAPIView.enroll_status_code),

    # 报名统计
    re_path(r'^statistics/?$', EnrollStatisticsAPI.statistics),
    re_path(r'^statistics_by_user/?$', EnrollStatisticsAPI.statistics_by_user),

    # 报名状态接口
    re_path(r'^batch_edit_code/?$', EnrollStatusCodeAPI.batch_edit_code),
]



from django.db import models
from django.utils import timezone
from django.utils.timezone import now


class EnrollStatusCode(models.Model):
    """ Enroll_EnrollRuleValuate 报名状态表 [1:N]"""

    class Meta:
        db_table = 'enroll_status_code'
        verbose_name_plural = '报名状态表'

    id = models.AutoField(verbose_name='ID', primary_key=True, null=False, auto_created=True)
    enroll_status_code = models.IntegerField(verbose_name='状态code', blank=True, null=True)
    value = models.CharField(verbose_name='状态值', max_length=255, blank=True, null=True)
    config = models.JSONField(verbose_name='配置', blank=True, null=True)

    def __str__(self):
        return f"{self.value}"


class EnrollRuleGroup(models.Model):
    """ Enroll_EnrollRuleGroup 报名规则组表表 """

    class Meta:
        db_table = 'enroll_rule_group'
        verbose_name_plural = '报名规则组表表'

    id = models.BigAutoField(verbose_name='报名规则组ID', primary_key=True, null=False, auto_created=True, help_text='')
    rule_group = models.CharField(verbose_name='规则组', max_length=255, blank=True, null=True, help_text='')
    category_id = models.IntegerField(verbose_name='类别ID', blank=True, null=True, help_text='')
    classify_id = models.IntegerField(verbose_name='分类ID', blank=True, null=True, help_text='')
    description = models.CharField(verbose_name='描述', max_length=255, blank=True, null=True, help_text='')

    def __str__(self):
        return f"{self.rule_group}"


class EnrollRuleValuate(models.Model):
    """ Enroll_EnrollRuleValuate 报名规则计价表 [1:N]"""

    class Meta:
        db_table = 'enroll_rule_valuate'
        verbose_name_plural = '报名规则计价表'

    type_choices = [("enroll", 'enroll（报名主表）'), ("enroll_subitem", "enroll_subitem（报名分享表）")]
    field_choices = [
        ("thread_id", "thread_id(信息ID)"),
        ("category_id", "category_id(信息分类ID)"),
        ("user_id", "user_id(发布用户ID)"),
        ("trading_relate", "trading_relate(买卖关系)"),
        ("region_code", "region_code(行政区划编码)"),
        ("occupy_room", "occupy_room(占用房间号)"),
        ("enroll_status_code", "enroll_status_code(报名状态码)"),
        ("min_number", "min_number(最小起报人数)"),
        ("max_number", "max_number(最大报名人数)"),
        ("min_count_apiece", "min_count_apiece(每人最小购买数)"),
        ("max_count_apiece", "max_count_apiece(每人最大购买数)"),
        ("enroll_rule_group", "enroll_rule_group(报名规则组)"),
        ("price", "price(单价)"),
        ("count", "count(总数量)"),
        ("unit", "unit(单位)"),
        ("fee", "fee(小费)"),
        ("reduction", "reduction(商家减免)"),
        ("subitems_amount", "subitems_amount(分项合计)"),
        ("amount", "amount(总收)"),
        ("paid_amount", "paid_amount(已收)"),
        ("unpaid_amount", "unpaid_amount(未收)"),
        ("commision", "commision(佣金)"),
        ("deposit", "deposit(每份预付押金)"),
        ("hide_price", "hide_price(匿价)"),
        ("hide_user", "hide_user(匿名)"),
        ("has_repeat", "has_repeat(有重复报名)"),
        ("has_subitem", "has_subitem(有分项)"),
        ("has_audit", "has_audit(有审核)"),
        ("need_vouch", "need_vouch(需三方担保)"),
        ("need_deposit", "need_deposit(需押金)"),
        ("need_imprest", "need_imprest(需预付款)"),
        ("enable_pool", "enable_pool(启用报名池)"),
        ("pool_limit", "pool_limit(报名池容纳人数)"),
        ("pool_stopwatch", "pool_stopwatch(报名池倒计时)"),
        ("open_time", "open_time(报名开放时间)"),
        ("close_time", "close_time(报名关闭时间)"),
        ("launch_time", "launch_time(活动开始时间)"),
        ("finish_time", "finish_time(活动完成时间)"),
        ("spend_time", "spend_time(活动花费时间)"),
        ("create_time", "create_time(创建时间)"),
        ("update_time", "update_time(更新时间)"),
        ("snapshot", "snapshot(快照)"),
        ("remark", "remark(备注)"),
        ("subitem__name", "subitem__name(分项名称)"),
        ("subitem__price", "subitem__price(分项单价)"),
        ("subitem__count", "subitem__count(分项数量)"),
        ("subitem__unit", "subitem__unit(单位)"),
        ("subitem__amount", "subitem__amount(分项小计)"),
        ("subitem__description", "subitem__description(描述)"),
    ]
    id = models.AutoField(verbose_name='单项计价ID', primary_key=True, null=False, auto_created=True, help_text='')
    enroll_rule_group = models.ForeignKey(EnrollRuleGroup, verbose_name='报名规则组ID', db_column='enroll_rule_group_id', related_name='+', unique=False, on_delete=models.DO_NOTHING, help_text='')
    name = models.CharField(verbose_name='计价名称', max_length=255, blank=True, null=True, help_text='')
    type = models.CharField(verbose_name='表类型', max_length=255, blank=True, null=True, help_text='', choices=type_choices)
    field = models.CharField(verbose_name='字段', max_length=255, blank=True, null=True, help_text='', choices=field_choices)
    expression_string = models.CharField(verbose_name='逻辑表达式', max_length=65535, blank=True, null=True, help_text='')
    sort = models.IntegerField(verbose_name='排序序号', blank=True, null=True, help_text='注意：计算顺序按照次序号降序排列')

    def __str__(self):
        return f"{self.id}"


class Enroll(models.Model):
    """ 1、Enroll_Enroll 报名主表 """

    class Meta:
        db_table = 'enroll_enroll'
        verbose_name_plural = '1. 报名主表'

    trading_choice = [(0, 'None 未知'), (1, 'Sell 卖家'), (2, 'Buy 买家'), (3, 'Cooperate 合资'), ]
    bool_choice = [(0, '否'), (1, "是")]

    id = models.BigAutoField(verbose_name='ID', primary_key=True, null=False, auto_created=True, help_text='')
    thread_id = models.BigIntegerField(verbose_name='信息ID', blank=False, null=False, help_text='')
    category_id = models.IntegerField(verbose_name='信息分类ID', blank=False, null=False, help_text='')
    user_id = models.BigIntegerField(verbose_name='发布用户ID', blank=False, null=False, help_text='')
    trading_relate = models.IntegerField(verbose_name='买卖关系', choices=trading_choice, blank=True, null=True, help_text='1: Sell 卖家, 2: Buy 买家, 3: Cooperate 合资')
    region_code = models.CharField(verbose_name='行政区划编码', max_length=20, blank=True, null=True, help_text='详见GB/T2260《中华人民共和国行政区划代码》、GB/T10114《县以下行政区划代码编制规则》')
    occupy_room = models.CharField(verbose_name='占用房间号', max_length=255, blank=True, null=True, help_text='如填写占用房间号，则同一时段内不允许重复占用房间号')
    enroll_status_code = models.IntegerField(verbose_name='报名状态码', default=1, help_text='尚未开始/报名中/已结束等14个状态码', )
    min_number = models.IntegerField(verbose_name='最小起报人数', blank=True, null=True, help_text='', )
    max_number = models.IntegerField(verbose_name='最大报名人数', blank=True, null=True, help_text='')
    min_count_apiece = models.IntegerField(verbose_name='每人最小购买数', blank=True, null=True, help_text='')
    max_count_apiece = models.IntegerField(verbose_name='每人最大购买数', blank=True, null=True, help_text='')
    enroll_rule_group = models.ForeignKey(verbose_name='报名规则组', to=EnrollRuleValuate, db_column='enroll_rule_group_id', related_name='+', unique=False, blank=True, null=True,
                                          on_delete=models.DO_NOTHING, help_text='')
    price = models.DecimalField(verbose_name='单价', max_digits=32, decimal_places=8, default=0, db_index=True, help_text='')
    count = models.IntegerField(verbose_name='总数量', blank=True, null=True, help_text='')
    unit = models.CharField(verbose_name='单位', max_length=255, blank=True, null=True, help_text='')
    fee = models.DecimalField(verbose_name='小费', max_digits=32, decimal_places=2, db_index=True, blank=True, null=True, help_text='')
    reduction = models.DecimalField(verbose_name='商家减免', max_digits=32, decimal_places=2, db_index=True, default=0, help_text='商家减免额')
    subitems_amount = models.DecimalField(verbose_name='分项合计', max_digits=32, decimal_places=2, db_index=True, default=0, help_text='')
    amount = models.DecimalField(verbose_name='总收/付金额', max_digits=32, decimal_places=2, db_index=True, blank=True, null=True, help_text='')
    paid_amount = models.DecimalField(verbose_name='已收/付金额', max_digits=32, decimal_places=2, db_index=True, blank=True, null=True, help_text='')
    unpaid_amount = models.DecimalField(verbose_name='未收/付金额', max_digits=32, decimal_places=2, db_index=True, blank=True, null=True, help_text='补差价')
    commision = models.DecimalField(verbose_name='佣金', max_digits=32, decimal_places=2, db_index=True, default=0, help_text='每份提成')
    deposit = models.DecimalField(verbose_name='每份预付押金', max_digits=32, decimal_places=8, default=0, db_index=True, help_text='')
    hide_price = models.IntegerField(verbose_name='匿价', blank=True, null=True, help_text='', choices=bool_choice, default=0)
    hide_user = models.IntegerField(verbose_name='匿名', blank=True, null=True, help_text='', choices=bool_choice, default=0)
    has_repeat = models.IntegerField(verbose_name='有重复报名', blank=True, null=True, help_text='', choices=bool_choice, default=0)
    has_subitem = models.IntegerField(verbose_name='有分项', blank=True, null=True, help_text='', choices=bool_choice, default=0)
    has_audit = models.IntegerField(verbose_name='有审核', blank=True, null=True, help_text='', choices=bool_choice, default=1)
    need_vouch = models.IntegerField(verbose_name='需三方担保', choices=bool_choice, default=1, blank=True, null=True, help_text='需三方担保')
    need_deposit = models.IntegerField(verbose_name='需押金', choices=bool_choice, default=1, blank=True, null=True, help_text='')
    need_imprest = models.IntegerField(verbose_name='需预付款', choices=bool_choice, default=1, blank=True, null=True, help_text='')
    enable_pool = models.IntegerField(verbose_name='启用报名池', choices=bool_choice, default=1, blank=True, null=True, help_text='')
    pool_limit = models.IntegerField(verbose_name='报名池容纳人数', blank=True, null=True, help_text='')
    pool_stopwatch = models.IntegerField(verbose_name='报名池倒计时', blank=True, null=True, help_text='单位（秒）')
    open_time = models.DateTimeField(verbose_name='报名开放时间', blank=True, null=True, help_text='')
    close_time = models.DateTimeField(verbose_name='报名关闭时间', blank=True, null=True, help_text='')
    launch_time = models.DateTimeField(verbose_name='活动开始时间', blank=True, null=True, help_text='')
    finish_time = models.DateTimeField(verbose_name='活动完成时间', blank=True, null=True, help_text='')
    spend_time = models.CharField(verbose_name='活动花费时间', max_length=255, blank=True, null=True, help_text='')
    create_time = models.DateTimeField(verbose_name='创建时间', default=timezone.now, help_text='')
    update_time = models.DateTimeField(verbose_name='更新时间', auto_now=True, help_text='')
    snapshot = models.JSONField(verbose_name='快照', blank=True, null=True, help_text='')
    remark = models.CharField(verbose_name='备注', max_length=255, blank=True, null=True, help_text='')

    def __str__(self):
        return f"{self.id}"


class EnrollExtendField(models.Model):
    """ EnrollExtendField 报名扩展字段表  临时使用"""

    class Meta:
        db_table = 'enroll_extend_field'
        verbose_name_plural = '报名分项表'

    id = models.AutoField(verbose_name='ID', primary_key=True, null=False, help_text='清单单价')
    field_index = models.IntegerField(verbose_name='原有字段', blank=True, null=True, help_text='')
    alias_name = models.CharField(verbose_name='映射名称', max_length=32, blank=True, null=True, help_text='')
    description = models.CharField(verbose_name='分项小计', max_length=255, blank=True, null=True, help_text='')


class EnrollSubitemExtendField(models.Model):
    """ EnrollSubitemExtendField 报名扩展字段表 """

    id = models.AutoField(verbose_name='ID', primary_key=True, null=False, help_text='清单单价')
    category_id = models.IntegerField(verbose_name='信息类别ID', blank=True, null=True, help_text='')
    field_index = models.CharField(verbose_name='映射的字段索引', max_length=32, blank=True, null=True, help_text='')
    field = models.CharField(verbose_name='自定义字段', max_length=32, blank=True, null=True, help_text='')
    label = models.CharField(verbose_name='字段标签', max_length=32, blank=True, null=True, help_text='')
    type = models.CharField(verbose_name='字段类型', max_length=32, blank=True, null=True, help_text='')
    config = models.JSONField(verbose_name='数据配置', blank=True, null=True, help_text='')
    description = models.CharField(verbose_name='描述', max_length=255, blank=True, null=True, help_text='')

    class Meta:
        db_table = 'enroll_subitem_extend_field'
        verbose_name_plural = '报名分项表'
        # 联合约束   其中goods和user不能重复
        unique_together = ["category_id", "field_index", "field"]
        # 联合索引
        index_together = ["category_id", "field_index", "field"]


class EnrollSubitem(models.Model):
    """ Enroll_EnrollSubitem 报名分项表 [1:N] """

    class Meta:
        db_table = 'enroll_subitem'
        verbose_name_plural = '报名分项表'

    id = models.BigAutoField(verbose_name='ID', primary_key=True, null=False, auto_created=True, help_text='报名分项ID')
    enroll = models.ForeignKey(to=Enroll, on_delete=models.DO_NOTHING, verbose_name='报名ID', blank=True, null=True, help_text='报名ID')
    name = models.CharField(verbose_name='分项名称', max_length=128, blank=True, null=True, help_text='')
    price = models.DecimalField(verbose_name='分项单价', max_digits=32, decimal_places=8, db_index=True, blank=True, null=True, help_text='清单单价')
    count = models.IntegerField(verbose_name='分项数量', blank=True, null=True, help_text='')
    unit = models.CharField(verbose_name='单位', max_length=32, blank=True, null=True, help_text='')
    amount = models.DecimalField(verbose_name='分项小计', max_digits=32, decimal_places=8, db_index=True, blank=True, null=True, help_text='')
    enroll_subitem_status_code = models.IntegerField(verbose_name='报名状态码', default=1, help_text='尚未开始/报名中/已结束等14个状态码', )
    description = models.CharField(verbose_name='描述', max_length=255, blank=True, null=True, help_text='')
    remark = models.CharField(verbose_name='备注', max_length=255, blank=True, null=True, help_text='')
    field_1 = models.CharField(verbose_name='字段1', max_length=255, blank=True, null=True, help_text='')
    field_2 = models.CharField(verbose_name='字段2', max_length=255, blank=True, null=True, help_text='')
    field_3 = models.CharField(verbose_name='字段3', max_length=255, blank=True, null=True, help_text='')
    field_4 = models.CharField(verbose_name='字段4', max_length=255, blank=True, null=True, help_text='')
    field_5 = models.CharField(verbose_name='字段5', max_length=255, blank=True, null=True, help_text='')
    field_6 = models.CharField(verbose_name='字段6', max_length=255, blank=True, null=True, help_text='')
    field_7 = models.CharField(verbose_name='字段7', max_length=255, blank=True, null=True, help_text='')
    field_8 = models.CharField(verbose_name='字段8', max_length=255, blank=True, null=True, help_text='')
    field_9 = models.CharField(verbose_name='字段9', max_length=255, blank=True, null=True, help_text='')
    field_10 = models.CharField(verbose_name='字段10', max_length=255, blank=True, null=True, help_text='')

    def __str__(self):
        return f"{self.id}"


class EnrollAuthStatus(models.Model):
    class Meta:
        db_table = 'enroll_auth_status'
        verbose_name_plural = '审核状态表'

    id = models.AutoField(verbose_name='ID', primary_key=True)
    value = models.CharField(verbose_name='值', max_length=255, blank=True, null=True)

    def __str__(self):
        return f"{self.value}"


class EnrollPayStatus(models.Model):
    class Meta:
        db_table = 'enroll_pay_status'
        verbose_name_plural = '支付状态表'

    id = models.AutoField(verbose_name='ID', primary_key=True)
    value = models.CharField(verbose_name='值', max_length=255, blank=True, null=True)

    def __str__(self):
        return f"{self.value}"


class EnrollRecord(models.Model):
    """ Enroll_EnrollRecord 报名记录表 [1:N] """

    class Meta:
        db_table = 'enroll_record'
        verbose_name_plural = '报名记录表'

    id = models.BigAutoField(verbose_name='报名记录ID', primary_key=True, null=False, auto_created=True, help_text='报名记录ID')
    enroll = models.ForeignKey(verbose_name='报名ID', to=Enroll, db_column='enroll_id', related_name='+', on_delete=models.DO_NOTHING)
    user_id = models.BigIntegerField(verbose_name='用户ID', db_index=True)
    enroll_auth_status = models.ForeignKey(verbose_name='报名权限状态ID', to=EnrollAuthStatus, db_column='enroll_auth_status_id', related_name='+', on_delete=models.DO_NOTHING, blank=True, null=True,
                                           help_text='')
    enroll_pay_status = models.ForeignKey(verbose_name='报名支付状态ID', to=EnrollPayStatus, db_column='enroll_pay_status_id', related_name='+', on_delete=models.DO_NOTHING, blank=True, null=True,
                                          help_text='')
    enroll_status_code = models.IntegerField(verbose_name='报名状态码', blank=True, null=True, help_text='')
    create_time = models.DateTimeField(verbose_name='报名时间', default=timezone.now, blank=True, null=True, help_text='')
    price = models.DecimalField(verbose_name='应付单价', max_digits=32, decimal_places=8, db_index=True, blank=True, null=True, default=0, help_text='')
    deposit = models.DecimalField(verbose_name='应付押金', max_digits=32, decimal_places=8, db_index=True, blank=True, null=True, default=0, help_text='')
    count = models.DecimalField(verbose_name='购买数量', max_digits=32, decimal_places=8, db_index=True, blank=True, null=True, default=0, help_text='')
    main_amount = models.DecimalField(verbose_name='主项合计', max_digits=32, decimal_places=8, db_index=True, blank=True, null=True, default=0, help_text='')
    coupon_amount = models.DecimalField(verbose_name='优惠券合计', max_digits=32, decimal_places=8, db_index=True, blank=True, null=True, default=0, help_text='')
    again_reduction = models.DecimalField(verbose_name='商家再减免', max_digits=32, decimal_places=8, db_index=True, blank=True, null=True, default=0, help_text='商家再减免(发起人填)')
    subitems_amount = models.DecimalField(verbose_name='分项合计', max_digits=32, decimal_places=8, db_index=True, blank=True, null=True, default=0, help_text='')
    deposit_amount = models.DecimalField(verbose_name='押金合计', max_digits=32, decimal_places=8, db_index=True, blank=True, null=True, default=0, help_text='')
    amount = models.DecimalField(verbose_name='总计应收/付金额', max_digits=32, decimal_places=8, db_index=True, blank=True, null=True, default=0, help_text='')
    paid_amount = models.DecimalField(verbose_name='已收/付金额', max_digits=32, decimal_places=8, db_index=True, blank=True, null=True, default=0, help_text='')
    unpaid_amount = models.DecimalField(verbose_name='未收/付金额', max_digits=32, decimal_places=8, db_index=True, blank=True, null=True, default=0, help_text='')
    fee = models.DecimalField(verbose_name='小费', max_digits=32, decimal_places=8, db_index=True, blank=True, null=True, default=0, help_text='')
    photos = models.JSONField(verbose_name='图册地址', blank=True, null=True, default=list, help_text='')
    files = models.JSONField(verbose_name='文件地址', blank=True, null=True, default=list, help_text='')
    score = models.FloatField(verbose_name='评分', blank=True, null=True, help_text='')
    reply = models.CharField(verbose_name='发起人答复', max_length=1024, blank=True, null=True, help_text='发起人答复(发起人填)')
    remark = models.CharField(verbose_name='备注', max_length=1024, blank=True, null=True, help_text='')
    finish_time = models.DateTimeField(verbose_name='完成时间', blank=True, null=True, help_text='')
    again_price = models.DecimalField(verbose_name='追加价格', max_digits=10, decimal_places=2, db_index=True, blank=True, null=True, default=0, help_text='')
    again_fee = models.DecimalField(verbose_name='追加消费', max_digits=10, decimal_places=2, db_index=True, blank=True, null=True, default=0, help_text='')
    initiator_again_price = models.DecimalField(verbose_name='发起人追补单价', max_digits=10, decimal_places=2, db_index=True, blank=True, null=True, default=0, help_text='')
    estimated_time = models.DateTimeField(verbose_name='预估（完成）时间', blank=True, null=True, help_text='')
    update_time = models.DateTimeField(verbose_name='预估（完成）时间', blank=True, null=False, help_text='', default=now())
    field_1 = models.CharField(verbose_name='字段1', max_length=255, blank=True, null=True, help_text='')
    field_2 = models.CharField(verbose_name='字段2', max_length=255, blank=True, null=True, help_text='')
    field_3 = models.CharField(verbose_name='字段3', max_length=255, blank=True, null=True, help_text='')
    field_4 = models.CharField(verbose_name='字段4', max_length=255, blank=True, null=True, help_text='')
    field_5 = models.CharField(verbose_name='字段5', max_length=255, blank=True, null=True, help_text='')
    field_6 = models.CharField(verbose_name='字段6', max_length=255, blank=True, null=True, help_text='')
    field_7 = models.CharField(verbose_name='字段7', max_length=255, blank=True, null=True, help_text='')
    field_8 = models.CharField(verbose_name='字段8', max_length=255, blank=True, null=True, help_text='')
    field_9 = models.CharField(verbose_name='字段9', max_length=255, blank=True, null=True, help_text='')
    field_10 = models.CharField(verbose_name='字段10', max_length=255, blank=True, null=True, help_text='')

    def __str__(self):
        return f"{self.id}"


class EnrollRecordExtendField(models.Model):
    """ EnrollSubitemExtendField 报名扩展字段表 """
    id = models.AutoField(verbose_name='ID', primary_key=True, null=False, help_text='清单单价')
    field_index = models.CharField(verbose_name='映射的字段索引', max_length=32, blank=True, null=True, help_text='')
    field = models.CharField(verbose_name='自定义字段', max_length=32, blank=True, null=True, help_text='')
    label = models.CharField(verbose_name='字段标签', max_length=32, blank=True, null=True, help_text='')
    type = models.CharField(verbose_name='字段类型', max_length=32, blank=True, null=True, help_text='')
    config = models.JSONField(verbose_name='数据配置', blank=True, null=True, help_text='')
    description = models.CharField(verbose_name='描述', max_length=255, blank=True, null=True, help_text='')

    class Meta:
        db_table = 'enroll_record_extend_field'
        verbose_name_plural = '报名分项表'
        # 联合约束   其中goods和user不能重复
        unique_together = ["field_index", "field"]
        # 联合索引
        index_together = ["field_index", "field"]


class EnrollSubitemRecord(models.Model):
    """ Enroll_EnrollRecordSubitem 报名记录分项表 [1:N] """

    class Meta:
        db_table = 'enroll_subitem_record'
        verbose_name_plural = '报名记录分项表'

    id = models.BigAutoField(verbose_name='报名记录分项ID', primary_key=True, null=False, auto_created=True, help_text='')
    enroll_record = models.ForeignKey(
        verbose_name='报名记录ID', to=EnrollRecord, db_column='enroll_record_id', related_name='+', unique=False, blank=True, null=True, on_delete=models.DO_NOTHING, help_text=''
    )
    enroll_subitem = models.ForeignKey(
        verbose_name='报名分项ID', to=EnrollSubitem, db_column='enroll_subitem_id', related_name='+',
        unique=False, blank=True, null=True, on_delete=models.DO_NOTHING,
        help_text='说明：如果分项选择失败，行业规定是整单取消重新购买，不支持选择某项来取消。但为了灵活，允许后台工作端修改每项单价和数量（分项小计不可以修改），也能达到取消的效果。'
    )
    enroll_subitem_status_code = models.IntegerField(verbose_name="报名分项状态码", blank=True, null=True)
    user_id = models.BigIntegerField(verbose_name='用户ID', blank=True, null=True, help_text='')
    price = models.DecimalField(verbose_name='分项单价', max_digits=32, decimal_places=8, db_index=True, blank=True, null=True, default=0, help_text='')
    count = models.DecimalField(verbose_name='分项数量', max_digits=32, decimal_places=8, db_index=True, blank=True, null=True, default=0, help_text='')
    subitem_amount = models.DecimalField(verbose_name='分项小计', max_digits=32, decimal_places=8, db_index=True, blank=True, null=True, default=0, help_text='')
    reply = models.CharField(verbose_name='填写回复原因', max_length=255, blank=True, null=True, help_text='')
    remark = models.CharField(verbose_name='备注', max_length=255, blank=True, null=True, help_text='')
    files = models.JSONField(verbose_name='文件字典', blank=True, null=True, help_text='')
    photos = models.JSONField(verbose_name='图片字典', blank=True, null=True, help_text='')

    def __str__(self):
        return f"{self.id}"



# encoding: utf-8
"""
@project: djangoModel->enroll_validator
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 报名验证器
@created_time: 2022/9/17 15:50
"""

from django import forms
from django.core.exceptions import ValidationError

from xj_thread.services.thread_item_service import ThreadItemService
from ..utils.validator import Validator


class EnrollValidator(Validator):
    # 信息ID验证
    def clean_thread_id(self):
        cleaned_data = super().clean()
        thread_id = cleaned_data.get('thread_id')
        data, err = ThreadItemService.detail(thread_id)
        if err:
            raise ValidationError('不存在该信息，请重新选择')

    # 报名截至时间验证
    def clean_close_time(self):
        cleaned_data = super().clean()
        open_time = cleaned_data.get('open_time')
        close_time = cleaned_data.get('close_time')
        if not open_time < close_time:
            raise forms.ValidationError('报名截至时间必须大于报名开始时间')

    # 报名截至时间验证
    def clean_finish_time(self):
        cleaned_data = super().clean()
        launch_time = cleaned_data.get('launch_time')
        finish_time = cleaned_data.get('finish_time')
        if not launch_time < finish_time:
            raise forms.ValidationError('报名截至时间必须大于报名开始时间')

    # ====================== 验证 =============================

    thread_id = forms.CharField(
        required=True,
        error_messages={
            "required": "信息ID必填 必填",
        },
    )

    # 报名开始与截至时间
    open_time = forms.DateTimeField(
        required=True,
        error_messages={
            "required": "open_time 必填 必填",
        },
    )
    close_time = forms.DateTimeField(
        required=True,
        error_messages={
            "required": "open_time 必填 必填",
        },
    )

    # 活动开始与截至时间
    launch_time = forms.CharField(
        required=True,
        error_messages={
            "required": "launch_time 必填 必填",
        },
    )
    finish_time = forms.CharField(
        required=True,
        error_messages={
            "required": "finish_time 必填 必填",
        },
    )

    # 报名人数西安至
    min_number = forms.IntegerField(
        required=True,
        error_messages={
            "required": "min_number 必填 必填",
        },
    )
    max_number = forms.IntegerField(
        required=True,
        error_messages={
            "required": "max_number 必填 必填",
        },
    )



# encoding: utf-8
"""
@project: djangoModel->j_valuation
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 计价服务类
@created_time: 2022/10/13 16:58
"""
import math
import re


# 计算器基类
class JBaseExpression:
    __result = None  # 结果
    __expression = None
    fun_type = ""

    # 获取所有子类
    @staticmethod
    def get_child_info():
        try:
            return {getattr(i, "name"): i for i in JBaseExpression.__subclasses__() if not getattr(i, "name", None) is None}, None
        except AttributeError:
            return {}, "扩展类name属性可以重复"

    # 解析变量
    @staticmethod
    def parse_variables(expression_string, input_dict):
        if expression_string is None:
            return "0"
        # 变量解析替换
        this_cell_value_match = re.compile("{{.*?}}").findall(str(expression_string))
        # 得到变量解析的键值对
        parsed_variable_map = {}
        for i in this_cell_value_match:
            parsed_variable_map.update({i: input_dict.get(i.replace("{{", "").replace("}}", ""), "0")})

        # 比那辆键值对替换
        for k, v in parsed_variable_map.items():
            expression_string = expression_string.replace(k, str(v) if v else "0")
        return expression_string, parsed_variable_map
        # 解析括号，成对解析

    @staticmethod
    def parsed_brackets(expression, need_list=False):
        twain_index_map = {}
        twain_index_list = []
        forward_index_list = []
        for char, index in zip(expression, range(len(expression))):
            if char == "(":
                forward_index_list.append(index)
            if char == ")":
                forward_bracket = forward_index_list.pop(-1)
                twain_index_map[forward_bracket] = (forward_bracket, index)
                twain_index_list.append((forward_bracket, index))
        return twain_index_list if need_list else twain_index_map

    # 检测是否可直接解析的公式，是否存在公式。
    def has_expression(self, expression_string):
        child_class, err = self.get_child_info()
        fun_patt = ""
        for name, instance in child_class.items():
            if not name:
                continue
            fun_patt = "(" + name + ")" if fun_patt == "" else fun_patt + "|" + "(" + name + ")"
        fun_patt = "(" + fun_patt + ")"

        return True if re.search(fun_patt, expression_string) else False

    # 解析公式
    def parse_expression_new(self, expression_string):
        child_class, err = self.get_child_info()
        # 公式匹配
        fun_patt = ""
        for name, instance in child_class.items():
            fun_patt = "(" + name + ")" if fun_patt == "" else fun_patt + "|" + "(" + name + ")"
        fun_patt = "(" + fun_patt + ")" + "?\([^\(]*?\)"
        match_string = re.search(fun_patt, expression_string)
        if not match_string:
            return None

        # 执行公式
        match_string = match_string.group()
        for name, instance in child_class.items():
            match_res = re.search(instance.patt, match_string)
            if match_res and match_res.group() == match_string:
                result = instance().process(match_string)
                expression_string = expression_string.replace(match_string, str(result))
                # print("expression_string:", expression_string, "result:", result, "name:", name)
                break

        # 递归解析
        if re.search("\([^\(]*?\)", expression_string):
            result = self.parse_expression_new(expression_string)
            expression_string = result if result else expression_string
        return expression_string

    # 解析公式 复杂度比较高 有优化的空间
    def parse_expression(self, expression_string):
        # 解析括号和函数
        brackets_twain_map = self.parsed_brackets(expression_string)

        # 解析函数
        replace_list = []
        child_class, err = self.get_child_info()
        for name, instance in child_class.items():
            if not name:
                continue
            for r in re.finditer(name, expression_string):
                name_start, name_end = r.span()
                bracket_start, bracket_end = brackets_twain_map[name_end]

                full_fun_string = expression_string[name_start:bracket_end + 1]
                fun_inner_string = expression_string[bracket_start:bracket_end + 1]
                # print("full_fun_string:", full_fun_string, "fun_inner_string:", fun_inner_string, "self.has_expression(fun_inner_string):", self.has_expression(fun_inner_string))
                if not self.has_expression(fun_inner_string):
                    result = instance().process(full_fun_string)
                    replace_list.append((full_fun_string, result))

        # 结果替换
        for full_fun_string, result in replace_list:
            expression_string = expression_string.replace(full_fun_string, str(result))

        if self.has_expression(expression_string):
            expression_string = self.parse_expression(expression_string)
        return expression_string


# 计算器 对外暴露的类
class JExpression(JBaseExpression):

    def process(self, expression_string):
        # 数据预处理
        if expression_string is None:
            return "", None
        if not len(re.findall("\(", expression_string)) == len(re.findall("\)", expression_string)):
            return None, "语法错误，括号应该成对存在"
        child_class, err = self.get_child_info()
        if err:
            return None, err
        expression_string = expression_string.upper().replace(" ", "").replace("=", "==").replace(">==", ">=").replace("<==", "<=")  # 公式字符串预处理

        # 解析公式
        # expression_string = self.parse_expression_new(expression_string)
        expression_string = self.parse_expression(expression_string)

        result = CalculateExpression().process(expression_string)
        return result, None


# ================================= 公式封装类 =======================================
#  基础计算
class CalculateExpression(JBaseExpression):
    name = ""
    patt = "\([^\(]*?\)"

    def process(self, expression_string):
        try:
            return eval(expression_string)  # 加加减乘除
        except ZeroDivisionError:
            return 0  # 0/n  的情况
        except Exception as e:
            # print("CalculateExpression e", str(e))
            return expression_string


# 布尔值判断、三元运算
class IFExpression(JBaseExpression):
    name = "IF"
    patt = "IF\([^\(]*?\)"

    def process(self, expression, *args, ):
        bool_expression, yes_expression, no_expression = expression.replace(IFExpression.name, "").replace("(", "").replace(")", "").split(",")
        if CalculateExpression().process(bool_expression):
            return yes_expression
        else:
            return no_expression


# 求和计算、（多参数在服务中获取）
class SUMExpression(JBaseExpression):
    name = "SUM"
    patt = "SUM\([^\(]*?\)"  # SUM(a,b,c) == a+b+c

    def process(self, expression):
        try:
            args = expression.replace(self.name, "").replace("(", "").replace(")", "").split(",")
            result = 0
            for i in args:
                result = eval(str(result) + i)
            return result
        except Exception as e:
            print(e)
            return 0


class CEILExpression(JBaseExpression):
    name = "CEIL"
    patt = "CEIL\([^\(]*?\)"  # ROUND(num, index, step)

    def process(self, expression):
        try:
            expression = expression.replace(self.name, "").replace("(", "").replace(")", "")
            return math.ceil(eval(expression))
        except Exception as e:
            print(e)
            return 0
